# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from ._enums import *

__all__ = ['CertificateArgs', 'Certificate']

@pulumi.input_type
class CertificateArgs:
    def __init__(__self__, *,
                 certificate_chain: Optional[pulumi.Input[_builtins.str]] = None,
                 domains: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 leaf_certificate: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 private_key: Optional[pulumi.Input[_builtins.str]] = None,
                 type: Optional[pulumi.Input[Union[_builtins.str, 'CertificateType']]] = None):
        """
        The set of arguments for constructing a Certificate resource.
        :param pulumi.Input[_builtins.str] certificate_chain: The full PEM-formatted trust chain
               between the certificate authority's certificate and your domain's TLS
               certificate. Only valid when type is `custom`.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] domains: List of fully qualified domain names (FQDNs) for
               which the certificate will be issued. The domains must be managed using
               DigitalOcean's DNS. Only valid when type is `lets_encrypt`.
        :param pulumi.Input[_builtins.str] leaf_certificate: The contents of a PEM-formatted public
               TLS certificate. Only valid when type is `custom`.
        :param pulumi.Input[_builtins.str] name: The name of the certificate for identification.
        :param pulumi.Input[_builtins.str] private_key: The contents of a PEM-formatted private-key
               corresponding to the SSL certificate. Only valid when type is `custom`.
        :param pulumi.Input[Union[_builtins.str, 'CertificateType']] type: The type of certificate to provision. Can be either
               `custom` or `lets_encrypt`. Defaults to `custom`.
        """
        if certificate_chain is not None:
            pulumi.set(__self__, "certificate_chain", certificate_chain)
        if domains is not None:
            pulumi.set(__self__, "domains", domains)
        if leaf_certificate is not None:
            pulumi.set(__self__, "leaf_certificate", leaf_certificate)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if private_key is not None:
            pulumi.set(__self__, "private_key", private_key)
        if type is not None:
            pulumi.set(__self__, "type", type)

    @_builtins.property
    @pulumi.getter(name="certificateChain")
    def certificate_chain(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The full PEM-formatted trust chain
        between the certificate authority's certificate and your domain's TLS
        certificate. Only valid when type is `custom`.
        """
        return pulumi.get(self, "certificate_chain")

    @certificate_chain.setter
    def certificate_chain(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "certificate_chain", value)

    @_builtins.property
    @pulumi.getter
    def domains(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        List of fully qualified domain names (FQDNs) for
        which the certificate will be issued. The domains must be managed using
        DigitalOcean's DNS. Only valid when type is `lets_encrypt`.
        """
        return pulumi.get(self, "domains")

    @domains.setter
    def domains(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "domains", value)

    @_builtins.property
    @pulumi.getter(name="leafCertificate")
    def leaf_certificate(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The contents of a PEM-formatted public
        TLS certificate. Only valid when type is `custom`.
        """
        return pulumi.get(self, "leaf_certificate")

    @leaf_certificate.setter
    def leaf_certificate(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "leaf_certificate", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the certificate for identification.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="privateKey")
    def private_key(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The contents of a PEM-formatted private-key
        corresponding to the SSL certificate. Only valid when type is `custom`.
        """
        return pulumi.get(self, "private_key")

    @private_key.setter
    def private_key(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "private_key", value)

    @_builtins.property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[Union[_builtins.str, 'CertificateType']]]:
        """
        The type of certificate to provision. Can be either
        `custom` or `lets_encrypt`. Defaults to `custom`.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[Union[_builtins.str, 'CertificateType']]]):
        pulumi.set(self, "type", value)


@pulumi.input_type
class _CertificateState:
    def __init__(__self__, *,
                 certificate_chain: Optional[pulumi.Input[_builtins.str]] = None,
                 domains: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 leaf_certificate: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 not_after: Optional[pulumi.Input[_builtins.str]] = None,
                 private_key: Optional[pulumi.Input[_builtins.str]] = None,
                 sha1_fingerprint: Optional[pulumi.Input[_builtins.str]] = None,
                 state: Optional[pulumi.Input[_builtins.str]] = None,
                 type: Optional[pulumi.Input[Union[_builtins.str, 'CertificateType']]] = None,
                 uuid: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering Certificate resources.
        :param pulumi.Input[_builtins.str] certificate_chain: The full PEM-formatted trust chain
               between the certificate authority's certificate and your domain's TLS
               certificate. Only valid when type is `custom`.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] domains: List of fully qualified domain names (FQDNs) for
               which the certificate will be issued. The domains must be managed using
               DigitalOcean's DNS. Only valid when type is `lets_encrypt`.
        :param pulumi.Input[_builtins.str] leaf_certificate: The contents of a PEM-formatted public
               TLS certificate. Only valid when type is `custom`.
        :param pulumi.Input[_builtins.str] name: The name of the certificate for identification.
        :param pulumi.Input[_builtins.str] not_after: The expiration date of the certificate
        :param pulumi.Input[_builtins.str] private_key: The contents of a PEM-formatted private-key
               corresponding to the SSL certificate. Only valid when type is `custom`.
        :param pulumi.Input[_builtins.str] sha1_fingerprint: The SHA-1 fingerprint of the certificate
        :param pulumi.Input[Union[_builtins.str, 'CertificateType']] type: The type of certificate to provision. Can be either
               `custom` or `lets_encrypt`. Defaults to `custom`.
        :param pulumi.Input[_builtins.str] uuid: The UUID of the certificate
        """
        if certificate_chain is not None:
            pulumi.set(__self__, "certificate_chain", certificate_chain)
        if domains is not None:
            pulumi.set(__self__, "domains", domains)
        if leaf_certificate is not None:
            pulumi.set(__self__, "leaf_certificate", leaf_certificate)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if not_after is not None:
            pulumi.set(__self__, "not_after", not_after)
        if private_key is not None:
            pulumi.set(__self__, "private_key", private_key)
        if sha1_fingerprint is not None:
            pulumi.set(__self__, "sha1_fingerprint", sha1_fingerprint)
        if state is not None:
            pulumi.set(__self__, "state", state)
        if type is not None:
            pulumi.set(__self__, "type", type)
        if uuid is not None:
            pulumi.set(__self__, "uuid", uuid)

    @_builtins.property
    @pulumi.getter(name="certificateChain")
    def certificate_chain(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The full PEM-formatted trust chain
        between the certificate authority's certificate and your domain's TLS
        certificate. Only valid when type is `custom`.
        """
        return pulumi.get(self, "certificate_chain")

    @certificate_chain.setter
    def certificate_chain(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "certificate_chain", value)

    @_builtins.property
    @pulumi.getter
    def domains(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        List of fully qualified domain names (FQDNs) for
        which the certificate will be issued. The domains must be managed using
        DigitalOcean's DNS. Only valid when type is `lets_encrypt`.
        """
        return pulumi.get(self, "domains")

    @domains.setter
    def domains(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "domains", value)

    @_builtins.property
    @pulumi.getter(name="leafCertificate")
    def leaf_certificate(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The contents of a PEM-formatted public
        TLS certificate. Only valid when type is `custom`.
        """
        return pulumi.get(self, "leaf_certificate")

    @leaf_certificate.setter
    def leaf_certificate(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "leaf_certificate", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the certificate for identification.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="notAfter")
    def not_after(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The expiration date of the certificate
        """
        return pulumi.get(self, "not_after")

    @not_after.setter
    def not_after(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "not_after", value)

    @_builtins.property
    @pulumi.getter(name="privateKey")
    def private_key(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The contents of a PEM-formatted private-key
        corresponding to the SSL certificate. Only valid when type is `custom`.
        """
        return pulumi.get(self, "private_key")

    @private_key.setter
    def private_key(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "private_key", value)

    @_builtins.property
    @pulumi.getter(name="sha1Fingerprint")
    def sha1_fingerprint(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The SHA-1 fingerprint of the certificate
        """
        return pulumi.get(self, "sha1_fingerprint")

    @sha1_fingerprint.setter
    def sha1_fingerprint(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "sha1_fingerprint", value)

    @_builtins.property
    @pulumi.getter
    def state(self) -> Optional[pulumi.Input[_builtins.str]]:
        return pulumi.get(self, "state")

    @state.setter
    def state(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "state", value)

    @_builtins.property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[Union[_builtins.str, 'CertificateType']]]:
        """
        The type of certificate to provision. Can be either
        `custom` or `lets_encrypt`. Defaults to `custom`.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[Union[_builtins.str, 'CertificateType']]]):
        pulumi.set(self, "type", value)

    @_builtins.property
    @pulumi.getter
    def uuid(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The UUID of the certificate
        """
        return pulumi.get(self, "uuid")

    @uuid.setter
    def uuid(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "uuid", value)


@pulumi.type_token("digitalocean:index/certificate:Certificate")
class Certificate(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 certificate_chain: Optional[pulumi.Input[_builtins.str]] = None,
                 domains: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 leaf_certificate: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 private_key: Optional[pulumi.Input[_builtins.str]] = None,
                 type: Optional[pulumi.Input[Union[_builtins.str, 'CertificateType']]] = None,
                 __props__=None):
        """
        Provides a DigitalOcean Certificate resource that allows you to manage
        certificates for configuring TLS termination in Load Balancers.
        Certificates created with this resource can be referenced in your
        Load Balancer configuration via their ID. The certificate can either
        be a custom one provided by you or automatically generated one with
        Let's Encrypt.

        ## Example Usage

        ### Custom Certificate

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean
        import pulumi_std as std

        cert = digitalocean.Certificate("cert",
            name="custom-example",
            type=digitalocean.CertificateType.CUSTOM,
            private_key=std.file(input="/Users/myuser/certs/privkey.pem").result,
            leaf_certificate=std.file(input="/Users/myuser/certs/cert.pem").result,
            certificate_chain=std.file(input="/Users/myuser/certs/fullchain.pem").result)
        ```

        ### Let's Encrypt Certificate

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        cert = digitalocean.Certificate("cert",
            name="le-example",
            type=digitalocean.CertificateType.LETS_ENCRYPT,
            domains=["example.com"])
        ```

        ### Use with Other Resources

        Both custom and Let's Encrypt certificates can be used with other resources
        including the `LoadBalancer` and `Cdn` resources.

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        cert = digitalocean.Certificate("cert",
            name="le-example",
            type=digitalocean.CertificateType.LETS_ENCRYPT,
            domains=["example.com"])
        # Create a new Load Balancer with TLS termination
        public = digitalocean.LoadBalancer("public",
            name="secure-loadbalancer-1",
            region=digitalocean.Region.NYC3,
            droplet_tag="backend",
            forwarding_rules=[{
                "entry_port": 443,
                "entry_protocol": "https",
                "target_port": 80,
                "target_protocol": "http",
                "certificate_name": cert.name,
            }])
        ```

        ## Import

        Certificates can be imported using the certificate `name`, e.g.

        ```sh
        $ pulumi import digitalocean:index/certificate:Certificate mycertificate cert-01
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] certificate_chain: The full PEM-formatted trust chain
               between the certificate authority's certificate and your domain's TLS
               certificate. Only valid when type is `custom`.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] domains: List of fully qualified domain names (FQDNs) for
               which the certificate will be issued. The domains must be managed using
               DigitalOcean's DNS. Only valid when type is `lets_encrypt`.
        :param pulumi.Input[_builtins.str] leaf_certificate: The contents of a PEM-formatted public
               TLS certificate. Only valid when type is `custom`.
        :param pulumi.Input[_builtins.str] name: The name of the certificate for identification.
        :param pulumi.Input[_builtins.str] private_key: The contents of a PEM-formatted private-key
               corresponding to the SSL certificate. Only valid when type is `custom`.
        :param pulumi.Input[Union[_builtins.str, 'CertificateType']] type: The type of certificate to provision. Can be either
               `custom` or `lets_encrypt`. Defaults to `custom`.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[CertificateArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a DigitalOcean Certificate resource that allows you to manage
        certificates for configuring TLS termination in Load Balancers.
        Certificates created with this resource can be referenced in your
        Load Balancer configuration via their ID. The certificate can either
        be a custom one provided by you or automatically generated one with
        Let's Encrypt.

        ## Example Usage

        ### Custom Certificate

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean
        import pulumi_std as std

        cert = digitalocean.Certificate("cert",
            name="custom-example",
            type=digitalocean.CertificateType.CUSTOM,
            private_key=std.file(input="/Users/myuser/certs/privkey.pem").result,
            leaf_certificate=std.file(input="/Users/myuser/certs/cert.pem").result,
            certificate_chain=std.file(input="/Users/myuser/certs/fullchain.pem").result)
        ```

        ### Let's Encrypt Certificate

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        cert = digitalocean.Certificate("cert",
            name="le-example",
            type=digitalocean.CertificateType.LETS_ENCRYPT,
            domains=["example.com"])
        ```

        ### Use with Other Resources

        Both custom and Let's Encrypt certificates can be used with other resources
        including the `LoadBalancer` and `Cdn` resources.

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        cert = digitalocean.Certificate("cert",
            name="le-example",
            type=digitalocean.CertificateType.LETS_ENCRYPT,
            domains=["example.com"])
        # Create a new Load Balancer with TLS termination
        public = digitalocean.LoadBalancer("public",
            name="secure-loadbalancer-1",
            region=digitalocean.Region.NYC3,
            droplet_tag="backend",
            forwarding_rules=[{
                "entry_port": 443,
                "entry_protocol": "https",
                "target_port": 80,
                "target_protocol": "http",
                "certificate_name": cert.name,
            }])
        ```

        ## Import

        Certificates can be imported using the certificate `name`, e.g.

        ```sh
        $ pulumi import digitalocean:index/certificate:Certificate mycertificate cert-01
        ```

        :param str resource_name: The name of the resource.
        :param CertificateArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(CertificateArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 certificate_chain: Optional[pulumi.Input[_builtins.str]] = None,
                 domains: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 leaf_certificate: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 private_key: Optional[pulumi.Input[_builtins.str]] = None,
                 type: Optional[pulumi.Input[Union[_builtins.str, 'CertificateType']]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = CertificateArgs.__new__(CertificateArgs)

            __props__.__dict__["certificate_chain"] = certificate_chain
            __props__.__dict__["domains"] = domains
            __props__.__dict__["leaf_certificate"] = leaf_certificate
            __props__.__dict__["name"] = name
            __props__.__dict__["private_key"] = None if private_key is None else pulumi.Output.secret(private_key)
            __props__.__dict__["type"] = type
            __props__.__dict__["not_after"] = None
            __props__.__dict__["sha1_fingerprint"] = None
            __props__.__dict__["state"] = None
            __props__.__dict__["uuid"] = None
        secret_opts = pulumi.ResourceOptions(additional_secret_outputs=["privateKey"])
        opts = pulumi.ResourceOptions.merge(opts, secret_opts)
        super(Certificate, __self__).__init__(
            'digitalocean:index/certificate:Certificate',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            certificate_chain: Optional[pulumi.Input[_builtins.str]] = None,
            domains: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            leaf_certificate: Optional[pulumi.Input[_builtins.str]] = None,
            name: Optional[pulumi.Input[_builtins.str]] = None,
            not_after: Optional[pulumi.Input[_builtins.str]] = None,
            private_key: Optional[pulumi.Input[_builtins.str]] = None,
            sha1_fingerprint: Optional[pulumi.Input[_builtins.str]] = None,
            state: Optional[pulumi.Input[_builtins.str]] = None,
            type: Optional[pulumi.Input[Union[_builtins.str, 'CertificateType']]] = None,
            uuid: Optional[pulumi.Input[_builtins.str]] = None) -> 'Certificate':
        """
        Get an existing Certificate resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] certificate_chain: The full PEM-formatted trust chain
               between the certificate authority's certificate and your domain's TLS
               certificate. Only valid when type is `custom`.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] domains: List of fully qualified domain names (FQDNs) for
               which the certificate will be issued. The domains must be managed using
               DigitalOcean's DNS. Only valid when type is `lets_encrypt`.
        :param pulumi.Input[_builtins.str] leaf_certificate: The contents of a PEM-formatted public
               TLS certificate. Only valid when type is `custom`.
        :param pulumi.Input[_builtins.str] name: The name of the certificate for identification.
        :param pulumi.Input[_builtins.str] not_after: The expiration date of the certificate
        :param pulumi.Input[_builtins.str] private_key: The contents of a PEM-formatted private-key
               corresponding to the SSL certificate. Only valid when type is `custom`.
        :param pulumi.Input[_builtins.str] sha1_fingerprint: The SHA-1 fingerprint of the certificate
        :param pulumi.Input[Union[_builtins.str, 'CertificateType']] type: The type of certificate to provision. Can be either
               `custom` or `lets_encrypt`. Defaults to `custom`.
        :param pulumi.Input[_builtins.str] uuid: The UUID of the certificate
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _CertificateState.__new__(_CertificateState)

        __props__.__dict__["certificate_chain"] = certificate_chain
        __props__.__dict__["domains"] = domains
        __props__.__dict__["leaf_certificate"] = leaf_certificate
        __props__.__dict__["name"] = name
        __props__.__dict__["not_after"] = not_after
        __props__.__dict__["private_key"] = private_key
        __props__.__dict__["sha1_fingerprint"] = sha1_fingerprint
        __props__.__dict__["state"] = state
        __props__.__dict__["type"] = type
        __props__.__dict__["uuid"] = uuid
        return Certificate(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="certificateChain")
    def certificate_chain(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The full PEM-formatted trust chain
        between the certificate authority's certificate and your domain's TLS
        certificate. Only valid when type is `custom`.
        """
        return pulumi.get(self, "certificate_chain")

    @_builtins.property
    @pulumi.getter
    def domains(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        List of fully qualified domain names (FQDNs) for
        which the certificate will be issued. The domains must be managed using
        DigitalOcean's DNS. Only valid when type is `lets_encrypt`.
        """
        return pulumi.get(self, "domains")

    @_builtins.property
    @pulumi.getter(name="leafCertificate")
    def leaf_certificate(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The contents of a PEM-formatted public
        TLS certificate. Only valid when type is `custom`.
        """
        return pulumi.get(self, "leaf_certificate")

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Output[_builtins.str]:
        """
        The name of the certificate for identification.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="notAfter")
    def not_after(self) -> pulumi.Output[_builtins.str]:
        """
        The expiration date of the certificate
        """
        return pulumi.get(self, "not_after")

    @_builtins.property
    @pulumi.getter(name="privateKey")
    def private_key(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The contents of a PEM-formatted private-key
        corresponding to the SSL certificate. Only valid when type is `custom`.
        """
        return pulumi.get(self, "private_key")

    @_builtins.property
    @pulumi.getter(name="sha1Fingerprint")
    def sha1_fingerprint(self) -> pulumi.Output[_builtins.str]:
        """
        The SHA-1 fingerprint of the certificate
        """
        return pulumi.get(self, "sha1_fingerprint")

    @_builtins.property
    @pulumi.getter
    def state(self) -> pulumi.Output[_builtins.str]:
        return pulumi.get(self, "state")

    @_builtins.property
    @pulumi.getter
    def type(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The type of certificate to provision. Can be either
        `custom` or `lets_encrypt`. Defaults to `custom`.
        """
        return pulumi.get(self, "type")

    @_builtins.property
    @pulumi.getter
    def uuid(self) -> pulumi.Output[_builtins.str]:
        """
        The UUID of the certificate
        """
        return pulumi.get(self, "uuid")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetVpcNatGatewayResult',
    'AwaitableGetVpcNatGatewayResult',
    'get_vpc_nat_gateway',
    'get_vpc_nat_gateway_output',
]

@pulumi.output_type
class GetVpcNatGatewayResult:
    """
    A collection of values returned by getVpcNatGateway.
    """
    def __init__(__self__, created_at=None, egresses=None, icmp_timeout_seconds=None, id=None, name=None, region=None, size=None, state=None, tcp_timeout_seconds=None, type=None, udp_timeout_seconds=None, updated_at=None, vpcs=None):
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if egresses and not isinstance(egresses, list):
            raise TypeError("Expected argument 'egresses' to be a list")
        pulumi.set(__self__, "egresses", egresses)
        if icmp_timeout_seconds and not isinstance(icmp_timeout_seconds, int):
            raise TypeError("Expected argument 'icmp_timeout_seconds' to be a int")
        pulumi.set(__self__, "icmp_timeout_seconds", icmp_timeout_seconds)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if size and not isinstance(size, int):
            raise TypeError("Expected argument 'size' to be a int")
        pulumi.set(__self__, "size", size)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)
        if tcp_timeout_seconds and not isinstance(tcp_timeout_seconds, int):
            raise TypeError("Expected argument 'tcp_timeout_seconds' to be a int")
        pulumi.set(__self__, "tcp_timeout_seconds", tcp_timeout_seconds)
        if type and not isinstance(type, str):
            raise TypeError("Expected argument 'type' to be a str")
        pulumi.set(__self__, "type", type)
        if udp_timeout_seconds and not isinstance(udp_timeout_seconds, int):
            raise TypeError("Expected argument 'udp_timeout_seconds' to be a int")
        pulumi.set(__self__, "udp_timeout_seconds", udp_timeout_seconds)
        if updated_at and not isinstance(updated_at, str):
            raise TypeError("Expected argument 'updated_at' to be a str")
        pulumi.set(__self__, "updated_at", updated_at)
        if vpcs and not isinstance(vpcs, list):
            raise TypeError("Expected argument 'vpcs' to be a list")
        pulumi.set(__self__, "vpcs", vpcs)

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> _builtins.str:
        return pulumi.get(self, "created_at")

    @_builtins.property
    @pulumi.getter
    def egresses(self) -> Sequence['outputs.GetVpcNatGatewayEgressResult']:
        return pulumi.get(self, "egresses")

    @_builtins.property
    @pulumi.getter(name="icmpTimeoutSeconds")
    def icmp_timeout_seconds(self) -> _builtins.int:
        return pulumi.get(self, "icmp_timeout_seconds")

    @_builtins.property
    @pulumi.getter
    def id(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def region(self) -> _builtins.str:
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def size(self) -> _builtins.int:
        return pulumi.get(self, "size")

    @_builtins.property
    @pulumi.getter
    def state(self) -> _builtins.str:
        return pulumi.get(self, "state")

    @_builtins.property
    @pulumi.getter(name="tcpTimeoutSeconds")
    def tcp_timeout_seconds(self) -> _builtins.int:
        return pulumi.get(self, "tcp_timeout_seconds")

    @_builtins.property
    @pulumi.getter
    def type(self) -> _builtins.str:
        return pulumi.get(self, "type")

    @_builtins.property
    @pulumi.getter(name="udpTimeoutSeconds")
    def udp_timeout_seconds(self) -> _builtins.int:
        return pulumi.get(self, "udp_timeout_seconds")

    @_builtins.property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> _builtins.str:
        return pulumi.get(self, "updated_at")

    @_builtins.property
    @pulumi.getter
    def vpcs(self) -> Sequence['outputs.GetVpcNatGatewayVpcResult']:
        return pulumi.get(self, "vpcs")


class AwaitableGetVpcNatGatewayResult(GetVpcNatGatewayResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetVpcNatGatewayResult(
            created_at=self.created_at,
            egresses=self.egresses,
            icmp_timeout_seconds=self.icmp_timeout_seconds,
            id=self.id,
            name=self.name,
            region=self.region,
            size=self.size,
            state=self.state,
            tcp_timeout_seconds=self.tcp_timeout_seconds,
            type=self.type,
            udp_timeout_seconds=self.udp_timeout_seconds,
            updated_at=self.updated_at,
            vpcs=self.vpcs)


def get_vpc_nat_gateway(id: Optional[_builtins.str] = None,
                        name: Optional[_builtins.str] = None,
                        opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetVpcNatGatewayResult:
    """
    ## Example Usage

    Get the VPC NAT Gateway by name:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    my_imported_vpc_nat_gateway = digitalocean.get_vpc_nat_gateway(name=my_existing_vpc_nat_gateway["name"])
    ```

    Get the VPC NAT Gateway by ID:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    my_imported_vpc_nat_gateway = digitalocean.get_vpc_nat_gateway(id=my_existing_vpc_nat_gateway["id"])
    ```


    :param _builtins.str id: The ID of VPC NAT Gateway.
    :param _builtins.str name: The name of VPC NAT Gateway.
    """
    __args__ = dict()
    __args__['id'] = id
    __args__['name'] = name
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getVpcNatGateway:getVpcNatGateway', __args__, opts=opts, typ=GetVpcNatGatewayResult).value

    return AwaitableGetVpcNatGatewayResult(
        created_at=pulumi.get(__ret__, 'created_at'),
        egresses=pulumi.get(__ret__, 'egresses'),
        icmp_timeout_seconds=pulumi.get(__ret__, 'icmp_timeout_seconds'),
        id=pulumi.get(__ret__, 'id'),
        name=pulumi.get(__ret__, 'name'),
        region=pulumi.get(__ret__, 'region'),
        size=pulumi.get(__ret__, 'size'),
        state=pulumi.get(__ret__, 'state'),
        tcp_timeout_seconds=pulumi.get(__ret__, 'tcp_timeout_seconds'),
        type=pulumi.get(__ret__, 'type'),
        udp_timeout_seconds=pulumi.get(__ret__, 'udp_timeout_seconds'),
        updated_at=pulumi.get(__ret__, 'updated_at'),
        vpcs=pulumi.get(__ret__, 'vpcs'))
def get_vpc_nat_gateway_output(id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                               name: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                               opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetVpcNatGatewayResult]:
    """
    ## Example Usage

    Get the VPC NAT Gateway by name:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    my_imported_vpc_nat_gateway = digitalocean.get_vpc_nat_gateway(name=my_existing_vpc_nat_gateway["name"])
    ```

    Get the VPC NAT Gateway by ID:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    my_imported_vpc_nat_gateway = digitalocean.get_vpc_nat_gateway(id=my_existing_vpc_nat_gateway["id"])
    ```


    :param _builtins.str id: The ID of VPC NAT Gateway.
    :param _builtins.str name: The name of VPC NAT Gateway.
    """
    __args__ = dict()
    __args__['id'] = id
    __args__['name'] = name
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('digitalocean:index/getVpcNatGateway:getVpcNatGateway', __args__, opts=opts, typ=GetVpcNatGatewayResult)
    return __ret__.apply(lambda __response__: GetVpcNatGatewayResult(
        created_at=pulumi.get(__response__, 'created_at'),
        egresses=pulumi.get(__response__, 'egresses'),
        icmp_timeout_seconds=pulumi.get(__response__, 'icmp_timeout_seconds'),
        id=pulumi.get(__response__, 'id'),
        name=pulumi.get(__response__, 'name'),
        region=pulumi.get(__response__, 'region'),
        size=pulumi.get(__response__, 'size'),
        state=pulumi.get(__response__, 'state'),
        tcp_timeout_seconds=pulumi.get(__response__, 'tcp_timeout_seconds'),
        type=pulumi.get(__response__, 'type'),
        udp_timeout_seconds=pulumi.get(__response__, 'udp_timeout_seconds'),
        updated_at=pulumi.get(__response__, 'updated_at'),
        vpcs=pulumi.get(__response__, 'vpcs')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetTagsResult',
    'AwaitableGetTagsResult',
    'get_tags',
    'get_tags_output',
]

@pulumi.output_type
class GetTagsResult:
    """
    A collection of values returned by getTags.
    """
    def __init__(__self__, filters=None, id=None, sorts=None, tags=None):
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if sorts and not isinstance(sorts, list):
            raise TypeError("Expected argument 'sorts' to be a list")
        pulumi.set(__self__, "sorts", sorts)
        if tags and not isinstance(tags, list):
            raise TypeError("Expected argument 'tags' to be a list")
        pulumi.set(__self__, "tags", tags)

    @_builtins.property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetTagsFilterResult']]:
        return pulumi.get(self, "filters")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def sorts(self) -> Optional[Sequence['outputs.GetTagsSortResult']]:
        return pulumi.get(self, "sorts")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Sequence['outputs.GetTagsTagResult']:
        return pulumi.get(self, "tags")


class AwaitableGetTagsResult(GetTagsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetTagsResult(
            filters=self.filters,
            id=self.id,
            sorts=self.sorts,
            tags=self.tags)


def get_tags(filters: Optional[Sequence[Union['GetTagsFilterArgs', 'GetTagsFilterArgsDict']]] = None,
             sorts: Optional[Sequence[Union['GetTagsSortArgs', 'GetTagsSortArgsDict']]] = None,
             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetTagsResult:
    """
    Returns a list of tags in your DigitalOcean account, with the ability to
    filter and sort the results. If no filters are specified, all tags will be
    returned.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    list = digitalocean.get_tags(sorts=[{
        "key": "total_resource_count",
        "direction": "asc",
    }])
    pulumi.export("sortedTags", list.tags)
    ```


    :param Sequence[Union['GetTagsFilterArgs', 'GetTagsFilterArgsDict']] filters: Filter the results.
           The `filter` block is documented below.
    :param Sequence[Union['GetTagsSortArgs', 'GetTagsSortArgsDict']] sorts: Sort the results.
           The `sort` block is documented below.
    """
    __args__ = dict()
    __args__['filters'] = filters
    __args__['sorts'] = sorts
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getTags:getTags', __args__, opts=opts, typ=GetTagsResult).value

    return AwaitableGetTagsResult(
        filters=pulumi.get(__ret__, 'filters'),
        id=pulumi.get(__ret__, 'id'),
        sorts=pulumi.get(__ret__, 'sorts'),
        tags=pulumi.get(__ret__, 'tags'))
def get_tags_output(filters: Optional[pulumi.Input[Optional[Sequence[Union['GetTagsFilterArgs', 'GetTagsFilterArgsDict']]]]] = None,
                    sorts: Optional[pulumi.Input[Optional[Sequence[Union['GetTagsSortArgs', 'GetTagsSortArgsDict']]]]] = None,
                    opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetTagsResult]:
    """
    Returns a list of tags in your DigitalOcean account, with the ability to
    filter and sort the results. If no filters are specified, all tags will be
    returned.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    list = digitalocean.get_tags(sorts=[{
        "key": "total_resource_count",
        "direction": "asc",
    }])
    pulumi.export("sortedTags", list.tags)
    ```


    :param Sequence[Union['GetTagsFilterArgs', 'GetTagsFilterArgsDict']] filters: Filter the results.
           The `filter` block is documented below.
    :param Sequence[Union['GetTagsSortArgs', 'GetTagsSortArgsDict']] sorts: Sort the results.
           The `sort` block is documented below.
    """
    __args__ = dict()
    __args__['filters'] = filters
    __args__['sorts'] = sorts
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('digitalocean:index/getTags:getTags', __args__, opts=opts, typ=GetTagsResult)
    return __ret__.apply(lambda __response__: GetTagsResult(
        filters=pulumi.get(__response__, 'filters'),
        id=pulumi.get(__response__, 'id'),
        sorts=pulumi.get(__response__, 'sorts'),
        tags=pulumi.get(__response__, 'tags')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._enums import *
from ._inputs import *

__all__ = ['DatabaseClusterArgs', 'DatabaseCluster']

@pulumi.input_type
class DatabaseClusterArgs:
    def __init__(__self__, *,
                 engine: pulumi.Input[_builtins.str],
                 node_count: pulumi.Input[_builtins.int],
                 region: pulumi.Input[Union[_builtins.str, 'Region']],
                 size: pulumi.Input[Union[_builtins.str, 'DatabaseSlug']],
                 backup_restore: Optional[pulumi.Input['DatabaseClusterBackupRestoreArgs']] = None,
                 eviction_policy: Optional[pulumi.Input[_builtins.str]] = None,
                 maintenance_windows: Optional[pulumi.Input[Sequence[pulumi.Input['DatabaseClusterMaintenanceWindowArgs']]]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 private_network_uuid: Optional[pulumi.Input[_builtins.str]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 sql_mode: Optional[pulumi.Input[_builtins.str]] = None,
                 storage_size_mib: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 version: Optional[pulumi.Input[_builtins.str]] = None):
        """
        The set of arguments for constructing a DatabaseCluster resource.
        :param pulumi.Input[_builtins.str] engine: Database engine used by the cluster (ex. `pg` for PostgreSQL, `mysql` for MySQL, `valkey` for Valkey, `mongodb` for MongoDB, or `kafka` for Kafka).
        :param pulumi.Input[_builtins.int] node_count: Number of nodes that will be included in the cluster. For `kafka` clusters, this must be 3.
        :param pulumi.Input[Union[_builtins.str, 'Region']] region: DigitalOcean region where the cluster will reside.
        :param pulumi.Input[Union[_builtins.str, 'DatabaseSlug']] size: Database Droplet size associated with the cluster (ex. `db-s-1vcpu-1gb`). See the DigitalOcean API for a [list of valid size slugs](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_list_options).
        :param pulumi.Input[_builtins.str] eviction_policy: A string specifying the eviction policy for a Valkey cluster. Valid values are: `noeviction`, `allkeys_lru`, `allkeys_random`, `volatile_lru`, `volatile_random`, or `volatile_ttl`.
        :param pulumi.Input[Sequence[pulumi.Input['DatabaseClusterMaintenanceWindowArgs']]] maintenance_windows: Defines when the automatic maintenance should be performed for the database cluster.
        :param pulumi.Input[_builtins.str] name: The name of the database cluster.
        :param pulumi.Input[_builtins.str] private_network_uuid: The ID of the VPC where the database cluster will be located.
        :param pulumi.Input[_builtins.str] project_id: The ID of the project that the database cluster is assigned to. If excluded when creating a new database cluster, it will be assigned to your default project.
        :param pulumi.Input[_builtins.str] sql_mode: A comma separated string specifying the  SQL modes for a MySQL cluster.
        :param pulumi.Input[_builtins.str] storage_size_mib: Defines the disk size, in MiB, allocated to the cluster. This can be adjusted on MySQL and PostgreSQL clusters based on predefined ranges for each slug/droplet size.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A list of tag names to be applied to the database cluster.
        :param pulumi.Input[_builtins.str] version: Engine version used by the cluster (ex. `14` for PostgreSQL 14).
               When this value is changed, a call to the [Upgrade major Version for a Database](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_update_major_version) API operation is made with the new version.
        """
        pulumi.set(__self__, "engine", engine)
        pulumi.set(__self__, "node_count", node_count)
        pulumi.set(__self__, "region", region)
        pulumi.set(__self__, "size", size)
        if backup_restore is not None:
            pulumi.set(__self__, "backup_restore", backup_restore)
        if eviction_policy is not None:
            pulumi.set(__self__, "eviction_policy", eviction_policy)
        if maintenance_windows is not None:
            pulumi.set(__self__, "maintenance_windows", maintenance_windows)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if private_network_uuid is not None:
            pulumi.set(__self__, "private_network_uuid", private_network_uuid)
        if project_id is not None:
            pulumi.set(__self__, "project_id", project_id)
        if sql_mode is not None:
            pulumi.set(__self__, "sql_mode", sql_mode)
        if storage_size_mib is not None:
            pulumi.set(__self__, "storage_size_mib", storage_size_mib)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if version is not None:
            pulumi.set(__self__, "version", version)

    @_builtins.property
    @pulumi.getter
    def engine(self) -> pulumi.Input[_builtins.str]:
        """
        Database engine used by the cluster (ex. `pg` for PostgreSQL, `mysql` for MySQL, `valkey` for Valkey, `mongodb` for MongoDB, or `kafka` for Kafka).
        """
        return pulumi.get(self, "engine")

    @engine.setter
    def engine(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "engine", value)

    @_builtins.property
    @pulumi.getter(name="nodeCount")
    def node_count(self) -> pulumi.Input[_builtins.int]:
        """
        Number of nodes that will be included in the cluster. For `kafka` clusters, this must be 3.
        """
        return pulumi.get(self, "node_count")

    @node_count.setter
    def node_count(self, value: pulumi.Input[_builtins.int]):
        pulumi.set(self, "node_count", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Input[Union[_builtins.str, 'Region']]:
        """
        DigitalOcean region where the cluster will reside.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: pulumi.Input[Union[_builtins.str, 'Region']]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def size(self) -> pulumi.Input[Union[_builtins.str, 'DatabaseSlug']]:
        """
        Database Droplet size associated with the cluster (ex. `db-s-1vcpu-1gb`). See the DigitalOcean API for a [list of valid size slugs](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_list_options).
        """
        return pulumi.get(self, "size")

    @size.setter
    def size(self, value: pulumi.Input[Union[_builtins.str, 'DatabaseSlug']]):
        pulumi.set(self, "size", value)

    @_builtins.property
    @pulumi.getter(name="backupRestore")
    def backup_restore(self) -> Optional[pulumi.Input['DatabaseClusterBackupRestoreArgs']]:
        return pulumi.get(self, "backup_restore")

    @backup_restore.setter
    def backup_restore(self, value: Optional[pulumi.Input['DatabaseClusterBackupRestoreArgs']]):
        pulumi.set(self, "backup_restore", value)

    @_builtins.property
    @pulumi.getter(name="evictionPolicy")
    def eviction_policy(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A string specifying the eviction policy for a Valkey cluster. Valid values are: `noeviction`, `allkeys_lru`, `allkeys_random`, `volatile_lru`, `volatile_random`, or `volatile_ttl`.
        """
        return pulumi.get(self, "eviction_policy")

    @eviction_policy.setter
    def eviction_policy(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "eviction_policy", value)

    @_builtins.property
    @pulumi.getter(name="maintenanceWindows")
    def maintenance_windows(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DatabaseClusterMaintenanceWindowArgs']]]]:
        """
        Defines when the automatic maintenance should be performed for the database cluster.
        """
        return pulumi.get(self, "maintenance_windows")

    @maintenance_windows.setter
    def maintenance_windows(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DatabaseClusterMaintenanceWindowArgs']]]]):
        pulumi.set(self, "maintenance_windows", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the database cluster.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="privateNetworkUuid")
    def private_network_uuid(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the VPC where the database cluster will be located.
        """
        return pulumi.get(self, "private_network_uuid")

    @private_network_uuid.setter
    def private_network_uuid(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "private_network_uuid", value)

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the project that the database cluster is assigned to. If excluded when creating a new database cluster, it will be assigned to your default project.
        """
        return pulumi.get(self, "project_id")

    @project_id.setter
    def project_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "project_id", value)

    @_builtins.property
    @pulumi.getter(name="sqlMode")
    def sql_mode(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A comma separated string specifying the  SQL modes for a MySQL cluster.
        """
        return pulumi.get(self, "sql_mode")

    @sql_mode.setter
    def sql_mode(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "sql_mode", value)

    @_builtins.property
    @pulumi.getter(name="storageSizeMib")
    def storage_size_mib(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Defines the disk size, in MiB, allocated to the cluster. This can be adjusted on MySQL and PostgreSQL clusters based on predefined ranges for each slug/droplet size.
        """
        return pulumi.get(self, "storage_size_mib")

    @storage_size_mib.setter
    def storage_size_mib(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "storage_size_mib", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A list of tag names to be applied to the database cluster.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter
    def version(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Engine version used by the cluster (ex. `14` for PostgreSQL 14).
        When this value is changed, a call to the [Upgrade major Version for a Database](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_update_major_version) API operation is made with the new version.
        """
        return pulumi.get(self, "version")

    @version.setter
    def version(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "version", value)


@pulumi.input_type
class _DatabaseClusterState:
    def __init__(__self__, *,
                 backup_restore: Optional[pulumi.Input['DatabaseClusterBackupRestoreArgs']] = None,
                 cluster_urn: Optional[pulumi.Input[_builtins.str]] = None,
                 database: Optional[pulumi.Input[_builtins.str]] = None,
                 engine: Optional[pulumi.Input[_builtins.str]] = None,
                 eviction_policy: Optional[pulumi.Input[_builtins.str]] = None,
                 host: Optional[pulumi.Input[_builtins.str]] = None,
                 maintenance_windows: Optional[pulumi.Input[Sequence[pulumi.Input['DatabaseClusterMaintenanceWindowArgs']]]] = None,
                 metrics_endpoints: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 node_count: Optional[pulumi.Input[_builtins.int]] = None,
                 password: Optional[pulumi.Input[_builtins.str]] = None,
                 port: Optional[pulumi.Input[_builtins.int]] = None,
                 private_host: Optional[pulumi.Input[_builtins.str]] = None,
                 private_network_uuid: Optional[pulumi.Input[_builtins.str]] = None,
                 private_uri: Optional[pulumi.Input[_builtins.str]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[Union[_builtins.str, 'Region']]] = None,
                 size: Optional[pulumi.Input[Union[_builtins.str, 'DatabaseSlug']]] = None,
                 sql_mode: Optional[pulumi.Input[_builtins.str]] = None,
                 storage_size_mib: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 ui_database: Optional[pulumi.Input[_builtins.str]] = None,
                 ui_host: Optional[pulumi.Input[_builtins.str]] = None,
                 ui_password: Optional[pulumi.Input[_builtins.str]] = None,
                 ui_port: Optional[pulumi.Input[_builtins.int]] = None,
                 ui_uri: Optional[pulumi.Input[_builtins.str]] = None,
                 ui_user: Optional[pulumi.Input[_builtins.str]] = None,
                 uri: Optional[pulumi.Input[_builtins.str]] = None,
                 user: Optional[pulumi.Input[_builtins.str]] = None,
                 version: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering DatabaseCluster resources.
        :param pulumi.Input[_builtins.str] cluster_urn: The uniform resource name of the database cluster.
        :param pulumi.Input[_builtins.str] database: Name of the cluster's default database.
        :param pulumi.Input[_builtins.str] engine: Database engine used by the cluster (ex. `pg` for PostgreSQL, `mysql` for MySQL, `valkey` for Valkey, `mongodb` for MongoDB, or `kafka` for Kafka).
        :param pulumi.Input[_builtins.str] eviction_policy: A string specifying the eviction policy for a Valkey cluster. Valid values are: `noeviction`, `allkeys_lru`, `allkeys_random`, `volatile_lru`, `volatile_random`, or `volatile_ttl`.
        :param pulumi.Input[_builtins.str] host: Database cluster's hostname.
        :param pulumi.Input[Sequence[pulumi.Input['DatabaseClusterMaintenanceWindowArgs']]] maintenance_windows: Defines when the automatic maintenance should be performed for the database cluster.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] metrics_endpoints: A list of metrics endpoints for the database cluster, providing URLs to access Prometheus-compatible metrics.
        :param pulumi.Input[_builtins.str] name: The name of the database cluster.
        :param pulumi.Input[_builtins.int] node_count: Number of nodes that will be included in the cluster. For `kafka` clusters, this must be 3.
        :param pulumi.Input[_builtins.str] password: Password for the cluster's default user.
        :param pulumi.Input[_builtins.int] port: Network port that the database cluster is listening on.
        :param pulumi.Input[_builtins.str] private_host: Same as `host`, but only accessible from resources within the account and in the same region.
        :param pulumi.Input[_builtins.str] private_network_uuid: The ID of the VPC where the database cluster will be located.
        :param pulumi.Input[_builtins.str] private_uri: Same as `uri`, but only accessible from resources within the account and in the same region.
        :param pulumi.Input[_builtins.str] project_id: The ID of the project that the database cluster is assigned to. If excluded when creating a new database cluster, it will be assigned to your default project.
        :param pulumi.Input[Union[_builtins.str, 'Region']] region: DigitalOcean region where the cluster will reside.
        :param pulumi.Input[Union[_builtins.str, 'DatabaseSlug']] size: Database Droplet size associated with the cluster (ex. `db-s-1vcpu-1gb`). See the DigitalOcean API for a [list of valid size slugs](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_list_options).
        :param pulumi.Input[_builtins.str] sql_mode: A comma separated string specifying the  SQL modes for a MySQL cluster.
        :param pulumi.Input[_builtins.str] storage_size_mib: Defines the disk size, in MiB, allocated to the cluster. This can be adjusted on MySQL and PostgreSQL clusters based on predefined ranges for each slug/droplet size.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A list of tag names to be applied to the database cluster.
        :param pulumi.Input[_builtins.str] ui_database: Name of the OpenSearch dashboard db.
        :param pulumi.Input[_builtins.str] ui_host: Hostname for the OpenSearch dashboard.
        :param pulumi.Input[_builtins.str] ui_password: Password for the OpenSearch dashboard's default user.
        :param pulumi.Input[_builtins.int] ui_port: Network port that the OpenSearch dashboard is listening on.
        :param pulumi.Input[_builtins.str] ui_uri: The full URI for connecting to the OpenSearch dashboard.
        :param pulumi.Input[_builtins.str] ui_user: Username for OpenSearch dashboard's default user.
        :param pulumi.Input[_builtins.str] uri: The full URI for connecting to the database cluster.
        :param pulumi.Input[_builtins.str] user: Username for the cluster's default user.
        :param pulumi.Input[_builtins.str] version: Engine version used by the cluster (ex. `14` for PostgreSQL 14).
               When this value is changed, a call to the [Upgrade major Version for a Database](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_update_major_version) API operation is made with the new version.
        """
        if backup_restore is not None:
            pulumi.set(__self__, "backup_restore", backup_restore)
        if cluster_urn is not None:
            pulumi.set(__self__, "cluster_urn", cluster_urn)
        if database is not None:
            pulumi.set(__self__, "database", database)
        if engine is not None:
            pulumi.set(__self__, "engine", engine)
        if eviction_policy is not None:
            pulumi.set(__self__, "eviction_policy", eviction_policy)
        if host is not None:
            pulumi.set(__self__, "host", host)
        if maintenance_windows is not None:
            pulumi.set(__self__, "maintenance_windows", maintenance_windows)
        if metrics_endpoints is not None:
            pulumi.set(__self__, "metrics_endpoints", metrics_endpoints)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if node_count is not None:
            pulumi.set(__self__, "node_count", node_count)
        if password is not None:
            pulumi.set(__self__, "password", password)
        if port is not None:
            pulumi.set(__self__, "port", port)
        if private_host is not None:
            pulumi.set(__self__, "private_host", private_host)
        if private_network_uuid is not None:
            pulumi.set(__self__, "private_network_uuid", private_network_uuid)
        if private_uri is not None:
            pulumi.set(__self__, "private_uri", private_uri)
        if project_id is not None:
            pulumi.set(__self__, "project_id", project_id)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if size is not None:
            pulumi.set(__self__, "size", size)
        if sql_mode is not None:
            pulumi.set(__self__, "sql_mode", sql_mode)
        if storage_size_mib is not None:
            pulumi.set(__self__, "storage_size_mib", storage_size_mib)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if ui_database is not None:
            pulumi.set(__self__, "ui_database", ui_database)
        if ui_host is not None:
            pulumi.set(__self__, "ui_host", ui_host)
        if ui_password is not None:
            pulumi.set(__self__, "ui_password", ui_password)
        if ui_port is not None:
            pulumi.set(__self__, "ui_port", ui_port)
        if ui_uri is not None:
            pulumi.set(__self__, "ui_uri", ui_uri)
        if ui_user is not None:
            pulumi.set(__self__, "ui_user", ui_user)
        if uri is not None:
            pulumi.set(__self__, "uri", uri)
        if user is not None:
            pulumi.set(__self__, "user", user)
        if version is not None:
            pulumi.set(__self__, "version", version)

    @_builtins.property
    @pulumi.getter(name="backupRestore")
    def backup_restore(self) -> Optional[pulumi.Input['DatabaseClusterBackupRestoreArgs']]:
        return pulumi.get(self, "backup_restore")

    @backup_restore.setter
    def backup_restore(self, value: Optional[pulumi.Input['DatabaseClusterBackupRestoreArgs']]):
        pulumi.set(self, "backup_restore", value)

    @_builtins.property
    @pulumi.getter(name="clusterUrn")
    def cluster_urn(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The uniform resource name of the database cluster.
        """
        return pulumi.get(self, "cluster_urn")

    @cluster_urn.setter
    def cluster_urn(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "cluster_urn", value)

    @_builtins.property
    @pulumi.getter
    def database(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Name of the cluster's default database.
        """
        return pulumi.get(self, "database")

    @database.setter
    def database(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "database", value)

    @_builtins.property
    @pulumi.getter
    def engine(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Database engine used by the cluster (ex. `pg` for PostgreSQL, `mysql` for MySQL, `valkey` for Valkey, `mongodb` for MongoDB, or `kafka` for Kafka).
        """
        return pulumi.get(self, "engine")

    @engine.setter
    def engine(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "engine", value)

    @_builtins.property
    @pulumi.getter(name="evictionPolicy")
    def eviction_policy(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A string specifying the eviction policy for a Valkey cluster. Valid values are: `noeviction`, `allkeys_lru`, `allkeys_random`, `volatile_lru`, `volatile_random`, or `volatile_ttl`.
        """
        return pulumi.get(self, "eviction_policy")

    @eviction_policy.setter
    def eviction_policy(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "eviction_policy", value)

    @_builtins.property
    @pulumi.getter
    def host(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Database cluster's hostname.
        """
        return pulumi.get(self, "host")

    @host.setter
    def host(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "host", value)

    @_builtins.property
    @pulumi.getter(name="maintenanceWindows")
    def maintenance_windows(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['DatabaseClusterMaintenanceWindowArgs']]]]:
        """
        Defines when the automatic maintenance should be performed for the database cluster.
        """
        return pulumi.get(self, "maintenance_windows")

    @maintenance_windows.setter
    def maintenance_windows(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['DatabaseClusterMaintenanceWindowArgs']]]]):
        pulumi.set(self, "maintenance_windows", value)

    @_builtins.property
    @pulumi.getter(name="metricsEndpoints")
    def metrics_endpoints(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A list of metrics endpoints for the database cluster, providing URLs to access Prometheus-compatible metrics.
        """
        return pulumi.get(self, "metrics_endpoints")

    @metrics_endpoints.setter
    def metrics_endpoints(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "metrics_endpoints", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the database cluster.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="nodeCount")
    def node_count(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Number of nodes that will be included in the cluster. For `kafka` clusters, this must be 3.
        """
        return pulumi.get(self, "node_count")

    @node_count.setter
    def node_count(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "node_count", value)

    @_builtins.property
    @pulumi.getter
    def password(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Password for the cluster's default user.
        """
        return pulumi.get(self, "password")

    @password.setter
    def password(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "password", value)

    @_builtins.property
    @pulumi.getter
    def port(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Network port that the database cluster is listening on.
        """
        return pulumi.get(self, "port")

    @port.setter
    def port(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "port", value)

    @_builtins.property
    @pulumi.getter(name="privateHost")
    def private_host(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Same as `host`, but only accessible from resources within the account and in the same region.
        """
        return pulumi.get(self, "private_host")

    @private_host.setter
    def private_host(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "private_host", value)

    @_builtins.property
    @pulumi.getter(name="privateNetworkUuid")
    def private_network_uuid(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the VPC where the database cluster will be located.
        """
        return pulumi.get(self, "private_network_uuid")

    @private_network_uuid.setter
    def private_network_uuid(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "private_network_uuid", value)

    @_builtins.property
    @pulumi.getter(name="privateUri")
    def private_uri(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Same as `uri`, but only accessible from resources within the account and in the same region.
        """
        return pulumi.get(self, "private_uri")

    @private_uri.setter
    def private_uri(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "private_uri", value)

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the project that the database cluster is assigned to. If excluded when creating a new database cluster, it will be assigned to your default project.
        """
        return pulumi.get(self, "project_id")

    @project_id.setter
    def project_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "project_id", value)

    @_builtins.property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[Union[_builtins.str, 'Region']]]:
        """
        DigitalOcean region where the cluster will reside.
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[Union[_builtins.str, 'Region']]]):
        pulumi.set(self, "region", value)

    @_builtins.property
    @pulumi.getter
    def size(self) -> Optional[pulumi.Input[Union[_builtins.str, 'DatabaseSlug']]]:
        """
        Database Droplet size associated with the cluster (ex. `db-s-1vcpu-1gb`). See the DigitalOcean API for a [list of valid size slugs](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_list_options).
        """
        return pulumi.get(self, "size")

    @size.setter
    def size(self, value: Optional[pulumi.Input[Union[_builtins.str, 'DatabaseSlug']]]):
        pulumi.set(self, "size", value)

    @_builtins.property
    @pulumi.getter(name="sqlMode")
    def sql_mode(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A comma separated string specifying the  SQL modes for a MySQL cluster.
        """
        return pulumi.get(self, "sql_mode")

    @sql_mode.setter
    def sql_mode(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "sql_mode", value)

    @_builtins.property
    @pulumi.getter(name="storageSizeMib")
    def storage_size_mib(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Defines the disk size, in MiB, allocated to the cluster. This can be adjusted on MySQL and PostgreSQL clusters based on predefined ranges for each slug/droplet size.
        """
        return pulumi.get(self, "storage_size_mib")

    @storage_size_mib.setter
    def storage_size_mib(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "storage_size_mib", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        A list of tag names to be applied to the database cluster.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter(name="uiDatabase")
    def ui_database(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Name of the OpenSearch dashboard db.
        """
        return pulumi.get(self, "ui_database")

    @ui_database.setter
    def ui_database(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "ui_database", value)

    @_builtins.property
    @pulumi.getter(name="uiHost")
    def ui_host(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Hostname for the OpenSearch dashboard.
        """
        return pulumi.get(self, "ui_host")

    @ui_host.setter
    def ui_host(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "ui_host", value)

    @_builtins.property
    @pulumi.getter(name="uiPassword")
    def ui_password(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Password for the OpenSearch dashboard's default user.
        """
        return pulumi.get(self, "ui_password")

    @ui_password.setter
    def ui_password(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "ui_password", value)

    @_builtins.property
    @pulumi.getter(name="uiPort")
    def ui_port(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Network port that the OpenSearch dashboard is listening on.
        """
        return pulumi.get(self, "ui_port")

    @ui_port.setter
    def ui_port(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "ui_port", value)

    @_builtins.property
    @pulumi.getter(name="uiUri")
    def ui_uri(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The full URI for connecting to the OpenSearch dashboard.
        """
        return pulumi.get(self, "ui_uri")

    @ui_uri.setter
    def ui_uri(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "ui_uri", value)

    @_builtins.property
    @pulumi.getter(name="uiUser")
    def ui_user(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Username for OpenSearch dashboard's default user.
        """
        return pulumi.get(self, "ui_user")

    @ui_user.setter
    def ui_user(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "ui_user", value)

    @_builtins.property
    @pulumi.getter
    def uri(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The full URI for connecting to the database cluster.
        """
        return pulumi.get(self, "uri")

    @uri.setter
    def uri(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "uri", value)

    @_builtins.property
    @pulumi.getter
    def user(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Username for the cluster's default user.
        """
        return pulumi.get(self, "user")

    @user.setter
    def user(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "user", value)

    @_builtins.property
    @pulumi.getter
    def version(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Engine version used by the cluster (ex. `14` for PostgreSQL 14).
        When this value is changed, a call to the [Upgrade major Version for a Database](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_update_major_version) API operation is made with the new version.
        """
        return pulumi.get(self, "version")

    @version.setter
    def version(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "version", value)


@pulumi.type_token("digitalocean:index/databaseCluster:DatabaseCluster")
class DatabaseCluster(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 backup_restore: Optional[pulumi.Input[Union['DatabaseClusterBackupRestoreArgs', 'DatabaseClusterBackupRestoreArgsDict']]] = None,
                 engine: Optional[pulumi.Input[_builtins.str]] = None,
                 eviction_policy: Optional[pulumi.Input[_builtins.str]] = None,
                 maintenance_windows: Optional[pulumi.Input[Sequence[pulumi.Input[Union['DatabaseClusterMaintenanceWindowArgs', 'DatabaseClusterMaintenanceWindowArgsDict']]]]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 node_count: Optional[pulumi.Input[_builtins.int]] = None,
                 private_network_uuid: Optional[pulumi.Input[_builtins.str]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[Union[_builtins.str, 'Region']]] = None,
                 size: Optional[pulumi.Input[Union[_builtins.str, 'DatabaseSlug']]] = None,
                 sql_mode: Optional[pulumi.Input[_builtins.str]] = None,
                 storage_size_mib: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 version: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        Provides a DigitalOcean database cluster resource.

        ## Example Usage

        ### Create a new PostgreSQL database cluster
        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        postgres_example = digitalocean.DatabaseCluster("postgres-example",
            name="example-postgres-cluster",
            engine="pg",
            version="15",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1,
            node_count=1)
        ```

        ### Create a new MySQL database cluster
        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        mysql_example = digitalocean.DatabaseCluster("mysql-example",
            name="example-mysql-cluster",
            engine="mysql",
            version="8",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1,
            node_count=1)
        ```

        ### Create a new Valkey database cluster
        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        valkey_example = digitalocean.DatabaseCluster("valkey-example",
            name="example-valkey-cluster",
            engine="valkey",
            version="8",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1,
            node_count=1)
        ```

        ### Create a new Kafka database cluster
        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        kafka_example = digitalocean.DatabaseCluster("kafka-example",
            name="example-kafka-cluster",
            engine="kafka",
            version="3.5",
            size="db-s-2vcpu-2gb",
            region=digitalocean.Region.NYC1,
            node_count=3)
        ```

        ### Create a new MongoDB database cluster
        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        mongodb_example = digitalocean.DatabaseCluster("mongodb-example",
            name="example-mongo-cluster",
            engine="mongodb",
            version="6",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC3,
            node_count=1)
        ```

        ### Create a new OpenSearch database cluster
        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        opensearch_example = digitalocean.DatabaseCluster("opensearch-example",
            name="example-opensearch-cluster",
            engine="opensearch",
            version="2",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU2_GB,
            region=digitalocean.Region.NYC3,
            node_count=1)
        ```

        ## Create a new database cluster based on a backup of an existing cluster.

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        doby = digitalocean.DatabaseCluster("doby",
            name="dobydb",
            engine="pg",
            version="15",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU2_GB,
            region=digitalocean.Region.NYC1,
            node_count=1,
            tags=["production"])
        doby_backup = digitalocean.DatabaseCluster("doby_backup",
            name="dobydupe",
            engine="pg",
            version="15",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU2_GB,
            region=digitalocean.Region.NYC1,
            node_count=1,
            tags=["production"],
            backup_restore={
                "database_name": "dobydb",
            },
            opts = pulumi.ResourceOptions(depends_on=[doby]))
        ```

        ## Import

        Database clusters can be imported using the `id` returned from DigitalOcean, e.g.

        ```sh
        $ pulumi import digitalocean:index/databaseCluster:DatabaseCluster mycluster 245bcfd0-7f31-4ce6-a2bc-475a116cca97
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] engine: Database engine used by the cluster (ex. `pg` for PostgreSQL, `mysql` for MySQL, `valkey` for Valkey, `mongodb` for MongoDB, or `kafka` for Kafka).
        :param pulumi.Input[_builtins.str] eviction_policy: A string specifying the eviction policy for a Valkey cluster. Valid values are: `noeviction`, `allkeys_lru`, `allkeys_random`, `volatile_lru`, `volatile_random`, or `volatile_ttl`.
        :param pulumi.Input[Sequence[pulumi.Input[Union['DatabaseClusterMaintenanceWindowArgs', 'DatabaseClusterMaintenanceWindowArgsDict']]]] maintenance_windows: Defines when the automatic maintenance should be performed for the database cluster.
        :param pulumi.Input[_builtins.str] name: The name of the database cluster.
        :param pulumi.Input[_builtins.int] node_count: Number of nodes that will be included in the cluster. For `kafka` clusters, this must be 3.
        :param pulumi.Input[_builtins.str] private_network_uuid: The ID of the VPC where the database cluster will be located.
        :param pulumi.Input[_builtins.str] project_id: The ID of the project that the database cluster is assigned to. If excluded when creating a new database cluster, it will be assigned to your default project.
        :param pulumi.Input[Union[_builtins.str, 'Region']] region: DigitalOcean region where the cluster will reside.
        :param pulumi.Input[Union[_builtins.str, 'DatabaseSlug']] size: Database Droplet size associated with the cluster (ex. `db-s-1vcpu-1gb`). See the DigitalOcean API for a [list of valid size slugs](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_list_options).
        :param pulumi.Input[_builtins.str] sql_mode: A comma separated string specifying the  SQL modes for a MySQL cluster.
        :param pulumi.Input[_builtins.str] storage_size_mib: Defines the disk size, in MiB, allocated to the cluster. This can be adjusted on MySQL and PostgreSQL clusters based on predefined ranges for each slug/droplet size.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A list of tag names to be applied to the database cluster.
        :param pulumi.Input[_builtins.str] version: Engine version used by the cluster (ex. `14` for PostgreSQL 14).
               When this value is changed, a call to the [Upgrade major Version for a Database](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_update_major_version) API operation is made with the new version.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: DatabaseClusterArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a DigitalOcean database cluster resource.

        ## Example Usage

        ### Create a new PostgreSQL database cluster
        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        postgres_example = digitalocean.DatabaseCluster("postgres-example",
            name="example-postgres-cluster",
            engine="pg",
            version="15",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1,
            node_count=1)
        ```

        ### Create a new MySQL database cluster
        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        mysql_example = digitalocean.DatabaseCluster("mysql-example",
            name="example-mysql-cluster",
            engine="mysql",
            version="8",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1,
            node_count=1)
        ```

        ### Create a new Valkey database cluster
        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        valkey_example = digitalocean.DatabaseCluster("valkey-example",
            name="example-valkey-cluster",
            engine="valkey",
            version="8",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC1,
            node_count=1)
        ```

        ### Create a new Kafka database cluster
        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        kafka_example = digitalocean.DatabaseCluster("kafka-example",
            name="example-kafka-cluster",
            engine="kafka",
            version="3.5",
            size="db-s-2vcpu-2gb",
            region=digitalocean.Region.NYC1,
            node_count=3)
        ```

        ### Create a new MongoDB database cluster
        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        mongodb_example = digitalocean.DatabaseCluster("mongodb-example",
            name="example-mongo-cluster",
            engine="mongodb",
            version="6",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU1_GB,
            region=digitalocean.Region.NYC3,
            node_count=1)
        ```

        ### Create a new OpenSearch database cluster
        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        opensearch_example = digitalocean.DatabaseCluster("opensearch-example",
            name="example-opensearch-cluster",
            engine="opensearch",
            version="2",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU2_GB,
            region=digitalocean.Region.NYC3,
            node_count=1)
        ```

        ## Create a new database cluster based on a backup of an existing cluster.

        ```python
        import pulumi
        import pulumi_digitalocean as digitalocean

        doby = digitalocean.DatabaseCluster("doby",
            name="dobydb",
            engine="pg",
            version="15",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU2_GB,
            region=digitalocean.Region.NYC1,
            node_count=1,
            tags=["production"])
        doby_backup = digitalocean.DatabaseCluster("doby_backup",
            name="dobydupe",
            engine="pg",
            version="15",
            size=digitalocean.DatabaseSlug.D_B_1_VPCU2_GB,
            region=digitalocean.Region.NYC1,
            node_count=1,
            tags=["production"],
            backup_restore={
                "database_name": "dobydb",
            },
            opts = pulumi.ResourceOptions(depends_on=[doby]))
        ```

        ## Import

        Database clusters can be imported using the `id` returned from DigitalOcean, e.g.

        ```sh
        $ pulumi import digitalocean:index/databaseCluster:DatabaseCluster mycluster 245bcfd0-7f31-4ce6-a2bc-475a116cca97
        ```

        :param str resource_name: The name of the resource.
        :param DatabaseClusterArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(DatabaseClusterArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 backup_restore: Optional[pulumi.Input[Union['DatabaseClusterBackupRestoreArgs', 'DatabaseClusterBackupRestoreArgsDict']]] = None,
                 engine: Optional[pulumi.Input[_builtins.str]] = None,
                 eviction_policy: Optional[pulumi.Input[_builtins.str]] = None,
                 maintenance_windows: Optional[pulumi.Input[Sequence[pulumi.Input[Union['DatabaseClusterMaintenanceWindowArgs', 'DatabaseClusterMaintenanceWindowArgsDict']]]]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 node_count: Optional[pulumi.Input[_builtins.int]] = None,
                 private_network_uuid: Optional[pulumi.Input[_builtins.str]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 region: Optional[pulumi.Input[Union[_builtins.str, 'Region']]] = None,
                 size: Optional[pulumi.Input[Union[_builtins.str, 'DatabaseSlug']]] = None,
                 sql_mode: Optional[pulumi.Input[_builtins.str]] = None,
                 storage_size_mib: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 version: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = DatabaseClusterArgs.__new__(DatabaseClusterArgs)

            __props__.__dict__["backup_restore"] = backup_restore
            if engine is None and not opts.urn:
                raise TypeError("Missing required property 'engine'")
            __props__.__dict__["engine"] = engine
            __props__.__dict__["eviction_policy"] = eviction_policy
            __props__.__dict__["maintenance_windows"] = maintenance_windows
            __props__.__dict__["name"] = name
            if node_count is None and not opts.urn:
                raise TypeError("Missing required property 'node_count'")
            __props__.__dict__["node_count"] = node_count
            __props__.__dict__["private_network_uuid"] = private_network_uuid
            __props__.__dict__["project_id"] = project_id
            if region is None and not opts.urn:
                raise TypeError("Missing required property 'region'")
            __props__.__dict__["region"] = region
            if size is None and not opts.urn:
                raise TypeError("Missing required property 'size'")
            __props__.__dict__["size"] = size
            __props__.__dict__["sql_mode"] = sql_mode
            __props__.__dict__["storage_size_mib"] = storage_size_mib
            __props__.__dict__["tags"] = tags
            __props__.__dict__["version"] = version
            __props__.__dict__["cluster_urn"] = None
            __props__.__dict__["database"] = None
            __props__.__dict__["host"] = None
            __props__.__dict__["metrics_endpoints"] = None
            __props__.__dict__["password"] = None
            __props__.__dict__["port"] = None
            __props__.__dict__["private_host"] = None
            __props__.__dict__["private_uri"] = None
            __props__.__dict__["ui_database"] = None
            __props__.__dict__["ui_host"] = None
            __props__.__dict__["ui_password"] = None
            __props__.__dict__["ui_port"] = None
            __props__.__dict__["ui_uri"] = None
            __props__.__dict__["ui_user"] = None
            __props__.__dict__["uri"] = None
            __props__.__dict__["user"] = None
        secret_opts = pulumi.ResourceOptions(additional_secret_outputs=["password", "privateUri", "uiPassword", "uiUri", "uri"])
        opts = pulumi.ResourceOptions.merge(opts, secret_opts)
        super(DatabaseCluster, __self__).__init__(
            'digitalocean:index/databaseCluster:DatabaseCluster',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            backup_restore: Optional[pulumi.Input[Union['DatabaseClusterBackupRestoreArgs', 'DatabaseClusterBackupRestoreArgsDict']]] = None,
            cluster_urn: Optional[pulumi.Input[_builtins.str]] = None,
            database: Optional[pulumi.Input[_builtins.str]] = None,
            engine: Optional[pulumi.Input[_builtins.str]] = None,
            eviction_policy: Optional[pulumi.Input[_builtins.str]] = None,
            host: Optional[pulumi.Input[_builtins.str]] = None,
            maintenance_windows: Optional[pulumi.Input[Sequence[pulumi.Input[Union['DatabaseClusterMaintenanceWindowArgs', 'DatabaseClusterMaintenanceWindowArgsDict']]]]] = None,
            metrics_endpoints: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            name: Optional[pulumi.Input[_builtins.str]] = None,
            node_count: Optional[pulumi.Input[_builtins.int]] = None,
            password: Optional[pulumi.Input[_builtins.str]] = None,
            port: Optional[pulumi.Input[_builtins.int]] = None,
            private_host: Optional[pulumi.Input[_builtins.str]] = None,
            private_network_uuid: Optional[pulumi.Input[_builtins.str]] = None,
            private_uri: Optional[pulumi.Input[_builtins.str]] = None,
            project_id: Optional[pulumi.Input[_builtins.str]] = None,
            region: Optional[pulumi.Input[Union[_builtins.str, 'Region']]] = None,
            size: Optional[pulumi.Input[Union[_builtins.str, 'DatabaseSlug']]] = None,
            sql_mode: Optional[pulumi.Input[_builtins.str]] = None,
            storage_size_mib: Optional[pulumi.Input[_builtins.str]] = None,
            tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            ui_database: Optional[pulumi.Input[_builtins.str]] = None,
            ui_host: Optional[pulumi.Input[_builtins.str]] = None,
            ui_password: Optional[pulumi.Input[_builtins.str]] = None,
            ui_port: Optional[pulumi.Input[_builtins.int]] = None,
            ui_uri: Optional[pulumi.Input[_builtins.str]] = None,
            ui_user: Optional[pulumi.Input[_builtins.str]] = None,
            uri: Optional[pulumi.Input[_builtins.str]] = None,
            user: Optional[pulumi.Input[_builtins.str]] = None,
            version: Optional[pulumi.Input[_builtins.str]] = None) -> 'DatabaseCluster':
        """
        Get an existing DatabaseCluster resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] cluster_urn: The uniform resource name of the database cluster.
        :param pulumi.Input[_builtins.str] database: Name of the cluster's default database.
        :param pulumi.Input[_builtins.str] engine: Database engine used by the cluster (ex. `pg` for PostgreSQL, `mysql` for MySQL, `valkey` for Valkey, `mongodb` for MongoDB, or `kafka` for Kafka).
        :param pulumi.Input[_builtins.str] eviction_policy: A string specifying the eviction policy for a Valkey cluster. Valid values are: `noeviction`, `allkeys_lru`, `allkeys_random`, `volatile_lru`, `volatile_random`, or `volatile_ttl`.
        :param pulumi.Input[_builtins.str] host: Database cluster's hostname.
        :param pulumi.Input[Sequence[pulumi.Input[Union['DatabaseClusterMaintenanceWindowArgs', 'DatabaseClusterMaintenanceWindowArgsDict']]]] maintenance_windows: Defines when the automatic maintenance should be performed for the database cluster.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] metrics_endpoints: A list of metrics endpoints for the database cluster, providing URLs to access Prometheus-compatible metrics.
        :param pulumi.Input[_builtins.str] name: The name of the database cluster.
        :param pulumi.Input[_builtins.int] node_count: Number of nodes that will be included in the cluster. For `kafka` clusters, this must be 3.
        :param pulumi.Input[_builtins.str] password: Password for the cluster's default user.
        :param pulumi.Input[_builtins.int] port: Network port that the database cluster is listening on.
        :param pulumi.Input[_builtins.str] private_host: Same as `host`, but only accessible from resources within the account and in the same region.
        :param pulumi.Input[_builtins.str] private_network_uuid: The ID of the VPC where the database cluster will be located.
        :param pulumi.Input[_builtins.str] private_uri: Same as `uri`, but only accessible from resources within the account and in the same region.
        :param pulumi.Input[_builtins.str] project_id: The ID of the project that the database cluster is assigned to. If excluded when creating a new database cluster, it will be assigned to your default project.
        :param pulumi.Input[Union[_builtins.str, 'Region']] region: DigitalOcean region where the cluster will reside.
        :param pulumi.Input[Union[_builtins.str, 'DatabaseSlug']] size: Database Droplet size associated with the cluster (ex. `db-s-1vcpu-1gb`). See the DigitalOcean API for a [list of valid size slugs](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_list_options).
        :param pulumi.Input[_builtins.str] sql_mode: A comma separated string specifying the  SQL modes for a MySQL cluster.
        :param pulumi.Input[_builtins.str] storage_size_mib: Defines the disk size, in MiB, allocated to the cluster. This can be adjusted on MySQL and PostgreSQL clusters based on predefined ranges for each slug/droplet size.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: A list of tag names to be applied to the database cluster.
        :param pulumi.Input[_builtins.str] ui_database: Name of the OpenSearch dashboard db.
        :param pulumi.Input[_builtins.str] ui_host: Hostname for the OpenSearch dashboard.
        :param pulumi.Input[_builtins.str] ui_password: Password for the OpenSearch dashboard's default user.
        :param pulumi.Input[_builtins.int] ui_port: Network port that the OpenSearch dashboard is listening on.
        :param pulumi.Input[_builtins.str] ui_uri: The full URI for connecting to the OpenSearch dashboard.
        :param pulumi.Input[_builtins.str] ui_user: Username for OpenSearch dashboard's default user.
        :param pulumi.Input[_builtins.str] uri: The full URI for connecting to the database cluster.
        :param pulumi.Input[_builtins.str] user: Username for the cluster's default user.
        :param pulumi.Input[_builtins.str] version: Engine version used by the cluster (ex. `14` for PostgreSQL 14).
               When this value is changed, a call to the [Upgrade major Version for a Database](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_update_major_version) API operation is made with the new version.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _DatabaseClusterState.__new__(_DatabaseClusterState)

        __props__.__dict__["backup_restore"] = backup_restore
        __props__.__dict__["cluster_urn"] = cluster_urn
        __props__.__dict__["database"] = database
        __props__.__dict__["engine"] = engine
        __props__.__dict__["eviction_policy"] = eviction_policy
        __props__.__dict__["host"] = host
        __props__.__dict__["maintenance_windows"] = maintenance_windows
        __props__.__dict__["metrics_endpoints"] = metrics_endpoints
        __props__.__dict__["name"] = name
        __props__.__dict__["node_count"] = node_count
        __props__.__dict__["password"] = password
        __props__.__dict__["port"] = port
        __props__.__dict__["private_host"] = private_host
        __props__.__dict__["private_network_uuid"] = private_network_uuid
        __props__.__dict__["private_uri"] = private_uri
        __props__.__dict__["project_id"] = project_id
        __props__.__dict__["region"] = region
        __props__.__dict__["size"] = size
        __props__.__dict__["sql_mode"] = sql_mode
        __props__.__dict__["storage_size_mib"] = storage_size_mib
        __props__.__dict__["tags"] = tags
        __props__.__dict__["ui_database"] = ui_database
        __props__.__dict__["ui_host"] = ui_host
        __props__.__dict__["ui_password"] = ui_password
        __props__.__dict__["ui_port"] = ui_port
        __props__.__dict__["ui_uri"] = ui_uri
        __props__.__dict__["ui_user"] = ui_user
        __props__.__dict__["uri"] = uri
        __props__.__dict__["user"] = user
        __props__.__dict__["version"] = version
        return DatabaseCluster(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="backupRestore")
    def backup_restore(self) -> pulumi.Output[Optional['outputs.DatabaseClusterBackupRestore']]:
        return pulumi.get(self, "backup_restore")

    @_builtins.property
    @pulumi.getter(name="clusterUrn")
    def cluster_urn(self) -> pulumi.Output[_builtins.str]:
        """
        The uniform resource name of the database cluster.
        """
        return pulumi.get(self, "cluster_urn")

    @_builtins.property
    @pulumi.getter
    def database(self) -> pulumi.Output[_builtins.str]:
        """
        Name of the cluster's default database.
        """
        return pulumi.get(self, "database")

    @_builtins.property
    @pulumi.getter
    def engine(self) -> pulumi.Output[_builtins.str]:
        """
        Database engine used by the cluster (ex. `pg` for PostgreSQL, `mysql` for MySQL, `valkey` for Valkey, `mongodb` for MongoDB, or `kafka` for Kafka).
        """
        return pulumi.get(self, "engine")

    @_builtins.property
    @pulumi.getter(name="evictionPolicy")
    def eviction_policy(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        A string specifying the eviction policy for a Valkey cluster. Valid values are: `noeviction`, `allkeys_lru`, `allkeys_random`, `volatile_lru`, `volatile_random`, or `volatile_ttl`.
        """
        return pulumi.get(self, "eviction_policy")

    @_builtins.property
    @pulumi.getter
    def host(self) -> pulumi.Output[_builtins.str]:
        """
        Database cluster's hostname.
        """
        return pulumi.get(self, "host")

    @_builtins.property
    @pulumi.getter(name="maintenanceWindows")
    def maintenance_windows(self) -> pulumi.Output[Optional[Sequence['outputs.DatabaseClusterMaintenanceWindow']]]:
        """
        Defines when the automatic maintenance should be performed for the database cluster.
        """
        return pulumi.get(self, "maintenance_windows")

    @_builtins.property
    @pulumi.getter(name="metricsEndpoints")
    def metrics_endpoints(self) -> pulumi.Output[Sequence[_builtins.str]]:
        """
        A list of metrics endpoints for the database cluster, providing URLs to access Prometheus-compatible metrics.
        """
        return pulumi.get(self, "metrics_endpoints")

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Output[_builtins.str]:
        """
        The name of the database cluster.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="nodeCount")
    def node_count(self) -> pulumi.Output[_builtins.int]:
        """
        Number of nodes that will be included in the cluster. For `kafka` clusters, this must be 3.
        """
        return pulumi.get(self, "node_count")

    @_builtins.property
    @pulumi.getter
    def password(self) -> pulumi.Output[_builtins.str]:
        """
        Password for the cluster's default user.
        """
        return pulumi.get(self, "password")

    @_builtins.property
    @pulumi.getter
    def port(self) -> pulumi.Output[_builtins.int]:
        """
        Network port that the database cluster is listening on.
        """
        return pulumi.get(self, "port")

    @_builtins.property
    @pulumi.getter(name="privateHost")
    def private_host(self) -> pulumi.Output[_builtins.str]:
        """
        Same as `host`, but only accessible from resources within the account and in the same region.
        """
        return pulumi.get(self, "private_host")

    @_builtins.property
    @pulumi.getter(name="privateNetworkUuid")
    def private_network_uuid(self) -> pulumi.Output[_builtins.str]:
        """
        The ID of the VPC where the database cluster will be located.
        """
        return pulumi.get(self, "private_network_uuid")

    @_builtins.property
    @pulumi.getter(name="privateUri")
    def private_uri(self) -> pulumi.Output[_builtins.str]:
        """
        Same as `uri`, but only accessible from resources within the account and in the same region.
        """
        return pulumi.get(self, "private_uri")

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> pulumi.Output[_builtins.str]:
        """
        The ID of the project that the database cluster is assigned to. If excluded when creating a new database cluster, it will be assigned to your default project.
        """
        return pulumi.get(self, "project_id")

    @_builtins.property
    @pulumi.getter
    def region(self) -> pulumi.Output[_builtins.str]:
        """
        DigitalOcean region where the cluster will reside.
        """
        return pulumi.get(self, "region")

    @_builtins.property
    @pulumi.getter
    def size(self) -> pulumi.Output[_builtins.str]:
        """
        Database Droplet size associated with the cluster (ex. `db-s-1vcpu-1gb`). See the DigitalOcean API for a [list of valid size slugs](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_list_options).
        """
        return pulumi.get(self, "size")

    @_builtins.property
    @pulumi.getter(name="sqlMode")
    def sql_mode(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        A comma separated string specifying the  SQL modes for a MySQL cluster.
        """
        return pulumi.get(self, "sql_mode")

    @_builtins.property
    @pulumi.getter(name="storageSizeMib")
    def storage_size_mib(self) -> pulumi.Output[_builtins.str]:
        """
        Defines the disk size, in MiB, allocated to the cluster. This can be adjusted on MySQL and PostgreSQL clusters based on predefined ranges for each slug/droplet size.
        """
        return pulumi.get(self, "storage_size_mib")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        A list of tag names to be applied to the database cluster.
        """
        return pulumi.get(self, "tags")

    @_builtins.property
    @pulumi.getter(name="uiDatabase")
    def ui_database(self) -> pulumi.Output[_builtins.str]:
        """
        Name of the OpenSearch dashboard db.
        """
        return pulumi.get(self, "ui_database")

    @_builtins.property
    @pulumi.getter(name="uiHost")
    def ui_host(self) -> pulumi.Output[_builtins.str]:
        """
        Hostname for the OpenSearch dashboard.
        """
        return pulumi.get(self, "ui_host")

    @_builtins.property
    @pulumi.getter(name="uiPassword")
    def ui_password(self) -> pulumi.Output[_builtins.str]:
        """
        Password for the OpenSearch dashboard's default user.
        """
        return pulumi.get(self, "ui_password")

    @_builtins.property
    @pulumi.getter(name="uiPort")
    def ui_port(self) -> pulumi.Output[_builtins.int]:
        """
        Network port that the OpenSearch dashboard is listening on.
        """
        return pulumi.get(self, "ui_port")

    @_builtins.property
    @pulumi.getter(name="uiUri")
    def ui_uri(self) -> pulumi.Output[_builtins.str]:
        """
        The full URI for connecting to the OpenSearch dashboard.
        """
        return pulumi.get(self, "ui_uri")

    @_builtins.property
    @pulumi.getter(name="uiUser")
    def ui_user(self) -> pulumi.Output[_builtins.str]:
        """
        Username for OpenSearch dashboard's default user.
        """
        return pulumi.get(self, "ui_user")

    @_builtins.property
    @pulumi.getter
    def uri(self) -> pulumi.Output[_builtins.str]:
        """
        The full URI for connecting to the database cluster.
        """
        return pulumi.get(self, "uri")

    @_builtins.property
    @pulumi.getter
    def user(self) -> pulumi.Output[_builtins.str]:
        """
        Username for the cluster's default user.
        """
        return pulumi.get(self, "user")

    @_builtins.property
    @pulumi.getter
    def version(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        Engine version used by the cluster (ex. `14` for PostgreSQL 14).
        When this value is changed, a call to the [Upgrade major Version for a Database](https://docs.digitalocean.com/reference/api/digitalocean/#tag/Databases/operation/databases_update_major_version) API operation is made with the new version.
        """
        return pulumi.get(self, "version")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetGenaiKnowledgeBasesResult',
    'AwaitableGetGenaiKnowledgeBasesResult',
    'get_genai_knowledge_bases',
    'get_genai_knowledge_bases_output',
]

@pulumi.output_type
class GetGenaiKnowledgeBasesResult:
    """
    A collection of values returned by getGenaiKnowledgeBases.
    """
    def __init__(__self__, filters=None, id=None, knowledge_bases=None, sorts=None):
        if filters and not isinstance(filters, list):
            raise TypeError("Expected argument 'filters' to be a list")
        pulumi.set(__self__, "filters", filters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if knowledge_bases and not isinstance(knowledge_bases, list):
            raise TypeError("Expected argument 'knowledge_bases' to be a list")
        pulumi.set(__self__, "knowledge_bases", knowledge_bases)
        if sorts and not isinstance(sorts, list):
            raise TypeError("Expected argument 'sorts' to be a list")
        pulumi.set(__self__, "sorts", sorts)

    @_builtins.property
    @pulumi.getter
    def filters(self) -> Optional[Sequence['outputs.GetGenaiKnowledgeBasesFilterResult']]:
        return pulumi.get(self, "filters")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="knowledgeBases")
    def knowledge_bases(self) -> Sequence['outputs.GetGenaiKnowledgeBasesKnowledgeBaseResult']:
        return pulumi.get(self, "knowledge_bases")

    @_builtins.property
    @pulumi.getter
    def sorts(self) -> Optional[Sequence['outputs.GetGenaiKnowledgeBasesSortResult']]:
        return pulumi.get(self, "sorts")


class AwaitableGetGenaiKnowledgeBasesResult(GetGenaiKnowledgeBasesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetGenaiKnowledgeBasesResult(
            filters=self.filters,
            id=self.id,
            knowledge_bases=self.knowledge_bases,
            sorts=self.sorts)


def get_genai_knowledge_bases(filters: Optional[Sequence[Union['GetGenaiKnowledgeBasesFilterArgs', 'GetGenaiKnowledgeBasesFilterArgsDict']]] = None,
                              sorts: Optional[Sequence[Union['GetGenaiKnowledgeBasesSortArgs', 'GetGenaiKnowledgeBasesSortArgsDict']]] = None,
                              opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetGenaiKnowledgeBasesResult:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['filters'] = filters
    __args__['sorts'] = sorts
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getGenaiKnowledgeBases:getGenaiKnowledgeBases', __args__, opts=opts, typ=GetGenaiKnowledgeBasesResult).value

    return AwaitableGetGenaiKnowledgeBasesResult(
        filters=pulumi.get(__ret__, 'filters'),
        id=pulumi.get(__ret__, 'id'),
        knowledge_bases=pulumi.get(__ret__, 'knowledge_bases'),
        sorts=pulumi.get(__ret__, 'sorts'))
def get_genai_knowledge_bases_output(filters: Optional[pulumi.Input[Optional[Sequence[Union['GetGenaiKnowledgeBasesFilterArgs', 'GetGenaiKnowledgeBasesFilterArgsDict']]]]] = None,
                                     sorts: Optional[pulumi.Input[Optional[Sequence[Union['GetGenaiKnowledgeBasesSortArgs', 'GetGenaiKnowledgeBasesSortArgsDict']]]]] = None,
                                     opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetGenaiKnowledgeBasesResult]:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['filters'] = filters
    __args__['sorts'] = sorts
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('digitalocean:index/getGenaiKnowledgeBases:getGenaiKnowledgeBases', __args__, opts=opts, typ=GetGenaiKnowledgeBasesResult)
    return __ret__.apply(lambda __response__: GetGenaiKnowledgeBasesResult(
        filters=pulumi.get(__response__, 'filters'),
        id=pulumi.get(__response__, 'id'),
        knowledge_bases=pulumi.get(__response__, 'knowledge_bases'),
        sorts=pulumi.get(__response__, 'sorts')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetGenaiOpenaiApiKeyResult',
    'AwaitableGetGenaiOpenaiApiKeyResult',
    'get_genai_openai_api_key',
    'get_genai_openai_api_key_output',
]

@pulumi.output_type
class GetGenaiOpenaiApiKeyResult:
    """
    A collection of values returned by getGenaiOpenaiApiKey.
    """
    def __init__(__self__, created_at=None, created_by=None, deleted_at=None, id=None, models=None, name=None, updated_at=None, uuid=None):
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if created_by and not isinstance(created_by, str):
            raise TypeError("Expected argument 'created_by' to be a str")
        pulumi.set(__self__, "created_by", created_by)
        if deleted_at and not isinstance(deleted_at, str):
            raise TypeError("Expected argument 'deleted_at' to be a str")
        pulumi.set(__self__, "deleted_at", deleted_at)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if models and not isinstance(models, list):
            raise TypeError("Expected argument 'models' to be a list")
        pulumi.set(__self__, "models", models)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if updated_at and not isinstance(updated_at, str):
            raise TypeError("Expected argument 'updated_at' to be a str")
        pulumi.set(__self__, "updated_at", updated_at)
        if uuid and not isinstance(uuid, str):
            raise TypeError("Expected argument 'uuid' to be a str")
        pulumi.set(__self__, "uuid", uuid)

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> _builtins.str:
        return pulumi.get(self, "created_at")

    @_builtins.property
    @pulumi.getter(name="createdBy")
    def created_by(self) -> _builtins.str:
        return pulumi.get(self, "created_by")

    @_builtins.property
    @pulumi.getter(name="deletedAt")
    def deleted_at(self) -> _builtins.str:
        return pulumi.get(self, "deleted_at")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter
    def models(self) -> Sequence['outputs.GetGenaiOpenaiApiKeyModelResult']:
        return pulumi.get(self, "models")

    @_builtins.property
    @pulumi.getter
    def name(self) -> _builtins.str:
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> _builtins.str:
        return pulumi.get(self, "updated_at")

    @_builtins.property
    @pulumi.getter
    def uuid(self) -> _builtins.str:
        return pulumi.get(self, "uuid")


class AwaitableGetGenaiOpenaiApiKeyResult(GetGenaiOpenaiApiKeyResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetGenaiOpenaiApiKeyResult(
            created_at=self.created_at,
            created_by=self.created_by,
            deleted_at=self.deleted_at,
            id=self.id,
            models=self.models,
            name=self.name,
            updated_at=self.updated_at,
            uuid=self.uuid)


def get_genai_openai_api_key(uuid: Optional[_builtins.str] = None,
                             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetGenaiOpenaiApiKeyResult:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['uuid'] = uuid
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getGenaiOpenaiApiKey:getGenaiOpenaiApiKey', __args__, opts=opts, typ=GetGenaiOpenaiApiKeyResult).value

    return AwaitableGetGenaiOpenaiApiKeyResult(
        created_at=pulumi.get(__ret__, 'created_at'),
        created_by=pulumi.get(__ret__, 'created_by'),
        deleted_at=pulumi.get(__ret__, 'deleted_at'),
        id=pulumi.get(__ret__, 'id'),
        models=pulumi.get(__ret__, 'models'),
        name=pulumi.get(__ret__, 'name'),
        updated_at=pulumi.get(__ret__, 'updated_at'),
        uuid=pulumi.get(__ret__, 'uuid'))
def get_genai_openai_api_key_output(uuid: Optional[pulumi.Input[_builtins.str]] = None,
                                    opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetGenaiOpenaiApiKeyResult]:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['uuid'] = uuid
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('digitalocean:index/getGenaiOpenaiApiKey:getGenaiOpenaiApiKey', __args__, opts=opts, typ=GetGenaiOpenaiApiKeyResult)
    return __ret__.apply(lambda __response__: GetGenaiOpenaiApiKeyResult(
        created_at=pulumi.get(__response__, 'created_at'),
        created_by=pulumi.get(__response__, 'created_by'),
        deleted_at=pulumi.get(__response__, 'deleted_at'),
        id=pulumi.get(__response__, 'id'),
        models=pulumi.get(__response__, 'models'),
        name=pulumi.get(__response__, 'name'),
        updated_at=pulumi.get(__response__, 'updated_at'),
        uuid=pulumi.get(__response__, 'uuid')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['ReservedIpv6AssignmentArgs', 'ReservedIpv6Assignment']

@pulumi.input_type
class ReservedIpv6AssignmentArgs:
    def __init__(__self__, *,
                 droplet_id: pulumi.Input[_builtins.int],
                 ip: pulumi.Input[_builtins.str]):
        """
        The set of arguments for constructing a ReservedIpv6Assignment resource.
        :param pulumi.Input[_builtins.int] droplet_id: The ID of Droplet that the reserved IPv6 will be assigned to.
        :param pulumi.Input[_builtins.str] ip: The reserved IPv6 to assign to the Droplet.
        """
        pulumi.set(__self__, "droplet_id", droplet_id)
        pulumi.set(__self__, "ip", ip)

    @_builtins.property
    @pulumi.getter(name="dropletId")
    def droplet_id(self) -> pulumi.Input[_builtins.int]:
        """
        The ID of Droplet that the reserved IPv6 will be assigned to.
        """
        return pulumi.get(self, "droplet_id")

    @droplet_id.setter
    def droplet_id(self, value: pulumi.Input[_builtins.int]):
        pulumi.set(self, "droplet_id", value)

    @_builtins.property
    @pulumi.getter
    def ip(self) -> pulumi.Input[_builtins.str]:
        """
        The reserved IPv6 to assign to the Droplet.
        """
        return pulumi.get(self, "ip")

    @ip.setter
    def ip(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "ip", value)


@pulumi.input_type
class _ReservedIpv6AssignmentState:
    def __init__(__self__, *,
                 droplet_id: Optional[pulumi.Input[_builtins.int]] = None,
                 ip: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering ReservedIpv6Assignment resources.
        :param pulumi.Input[_builtins.int] droplet_id: The ID of Droplet that the reserved IPv6 will be assigned to.
        :param pulumi.Input[_builtins.str] ip: The reserved IPv6 to assign to the Droplet.
        """
        if droplet_id is not None:
            pulumi.set(__self__, "droplet_id", droplet_id)
        if ip is not None:
            pulumi.set(__self__, "ip", ip)

    @_builtins.property
    @pulumi.getter(name="dropletId")
    def droplet_id(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The ID of Droplet that the reserved IPv6 will be assigned to.
        """
        return pulumi.get(self, "droplet_id")

    @droplet_id.setter
    def droplet_id(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "droplet_id", value)

    @_builtins.property
    @pulumi.getter
    def ip(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The reserved IPv6 to assign to the Droplet.
        """
        return pulumi.get(self, "ip")

    @ip.setter
    def ip(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "ip", value)


@pulumi.type_token("digitalocean:index/reservedIpv6Assignment:ReservedIpv6Assignment")
class ReservedIpv6Assignment(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 droplet_id: Optional[pulumi.Input[_builtins.int]] = None,
                 ip: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        Provides a resource for assigning an existing DigitalOcean reserved IPv6 to a Droplet. This
        makes it easy to provision reserved IPv6 addresses that are not tied to the lifecycle of your Droplet.

        ## Import

        Reserved IPv6 assignments can be imported using the reserved IPv6 itself and the `id` of

        the Droplet joined with a comma. For example:

        ```sh
        $ pulumi import digitalocean:index/reservedIpv6Assignment:ReservedIpv6Assignment foobar 2409:40d0:fa:27dd:9b24:7074:7b85:eee6,123456
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.int] droplet_id: The ID of Droplet that the reserved IPv6 will be assigned to.
        :param pulumi.Input[_builtins.str] ip: The reserved IPv6 to assign to the Droplet.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ReservedIpv6AssignmentArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a resource for assigning an existing DigitalOcean reserved IPv6 to a Droplet. This
        makes it easy to provision reserved IPv6 addresses that are not tied to the lifecycle of your Droplet.

        ## Import

        Reserved IPv6 assignments can be imported using the reserved IPv6 itself and the `id` of

        the Droplet joined with a comma. For example:

        ```sh
        $ pulumi import digitalocean:index/reservedIpv6Assignment:ReservedIpv6Assignment foobar 2409:40d0:fa:27dd:9b24:7074:7b85:eee6,123456
        ```

        :param str resource_name: The name of the resource.
        :param ReservedIpv6AssignmentArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ReservedIpv6AssignmentArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 droplet_id: Optional[pulumi.Input[_builtins.int]] = None,
                 ip: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ReservedIpv6AssignmentArgs.__new__(ReservedIpv6AssignmentArgs)

            if droplet_id is None and not opts.urn:
                raise TypeError("Missing required property 'droplet_id'")
            __props__.__dict__["droplet_id"] = droplet_id
            if ip is None and not opts.urn:
                raise TypeError("Missing required property 'ip'")
            __props__.__dict__["ip"] = ip
        super(ReservedIpv6Assignment, __self__).__init__(
            'digitalocean:index/reservedIpv6Assignment:ReservedIpv6Assignment',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            droplet_id: Optional[pulumi.Input[_builtins.int]] = None,
            ip: Optional[pulumi.Input[_builtins.str]] = None) -> 'ReservedIpv6Assignment':
        """
        Get an existing ReservedIpv6Assignment resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.int] droplet_id: The ID of Droplet that the reserved IPv6 will be assigned to.
        :param pulumi.Input[_builtins.str] ip: The reserved IPv6 to assign to the Droplet.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ReservedIpv6AssignmentState.__new__(_ReservedIpv6AssignmentState)

        __props__.__dict__["droplet_id"] = droplet_id
        __props__.__dict__["ip"] = ip
        return ReservedIpv6Assignment(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="dropletId")
    def droplet_id(self) -> pulumi.Output[_builtins.int]:
        """
        The ID of Droplet that the reserved IPv6 will be assigned to.
        """
        return pulumi.get(self, "droplet_id")

    @_builtins.property
    @pulumi.getter
    def ip(self) -> pulumi.Output[_builtins.str]:
        """
        The reserved IPv6 to assign to the Droplet.
        """
        return pulumi.get(self, "ip")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = [
    'GetFloatingIpResult',
    'AwaitableGetFloatingIpResult',
    'get_floating_ip',
    'get_floating_ip_output',
]

@pulumi.output_type
class GetFloatingIpResult:
    """
    A collection of values returned by getFloatingIp.
    """
    def __init__(__self__, droplet_id=None, floating_ip_urn=None, id=None, ip_address=None, region=None):
        if droplet_id and not isinstance(droplet_id, int):
            raise TypeError("Expected argument 'droplet_id' to be a int")
        pulumi.set(__self__, "droplet_id", droplet_id)
        if floating_ip_urn and not isinstance(floating_ip_urn, str):
            raise TypeError("Expected argument 'floating_ip_urn' to be a str")
        pulumi.set(__self__, "floating_ip_urn", floating_ip_urn)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ip_address and not isinstance(ip_address, str):
            raise TypeError("Expected argument 'ip_address' to be a str")
        pulumi.set(__self__, "ip_address", ip_address)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)

    @_builtins.property
    @pulumi.getter(name="dropletId")
    def droplet_id(self) -> _builtins.int:
        """
        The Droplet id that the floating IP has been assigned to.
        """
        return pulumi.get(self, "droplet_id")

    @_builtins.property
    @pulumi.getter(name="floatingIpUrn")
    def floating_ip_urn(self) -> _builtins.str:
        """
        The uniform resource name of the floating IP.
        """
        return pulumi.get(self, "floating_ip_urn")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="ipAddress")
    def ip_address(self) -> _builtins.str:
        return pulumi.get(self, "ip_address")

    @_builtins.property
    @pulumi.getter
    def region(self) -> _builtins.str:
        """
        The region that the floating IP is reserved to.
        """
        return pulumi.get(self, "region")


class AwaitableGetFloatingIpResult(GetFloatingIpResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetFloatingIpResult(
            droplet_id=self.droplet_id,
            floating_ip_urn=self.floating_ip_urn,
            id=self.id,
            ip_address=self.ip_address,
            region=self.region)


def get_floating_ip(ip_address: Optional[_builtins.str] = None,
                    opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetFloatingIpResult:
    """
    > **Deprecated:** DigitalOcean Floating IPs have been renamed reserved IPs. This data source will be removed in a future release. Please use `ReservedIp` instead.

    Get information on a floating ip. This data source provides the region and Droplet id
    as configured on your DigitalOcean account. This is useful if the floating IP
    in question is not managed by the provider or you need to find the Droplet the IP is
    attached to.

    An error is triggered if the provided floating IP does not exist.

    ## Example Usage

    Get the floating IP:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    config = pulumi.Config()
    public_ip = config.require_object("publicIp")
    example = digitalocean.get_floating_ip(ip_address=public_ip)
    pulumi.export("fipOutput", example.droplet_id)
    ```


    :param _builtins.str ip_address: The allocated IP address of the specific floating IP to retrieve.
    """
    __args__ = dict()
    __args__['ipAddress'] = ip_address
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('digitalocean:index/getFloatingIp:getFloatingIp', __args__, opts=opts, typ=GetFloatingIpResult).value

    return AwaitableGetFloatingIpResult(
        droplet_id=pulumi.get(__ret__, 'droplet_id'),
        floating_ip_urn=pulumi.get(__ret__, 'floating_ip_urn'),
        id=pulumi.get(__ret__, 'id'),
        ip_address=pulumi.get(__ret__, 'ip_address'),
        region=pulumi.get(__ret__, 'region'))
def get_floating_ip_output(ip_address: Optional[pulumi.Input[_builtins.str]] = None,
                           opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetFloatingIpResult]:
    """
    > **Deprecated:** DigitalOcean Floating IPs have been renamed reserved IPs. This data source will be removed in a future release. Please use `ReservedIp` instead.

    Get information on a floating ip. This data source provides the region and Droplet id
    as configured on your DigitalOcean account. This is useful if the floating IP
    in question is not managed by the provider or you need to find the Droplet the IP is
    attached to.

    An error is triggered if the provided floating IP does not exist.

    ## Example Usage

    Get the floating IP:

    ```python
    import pulumi
    import pulumi_digitalocean as digitalocean

    config = pulumi.Config()
    public_ip = config.require_object("publicIp")
    example = digitalocean.get_floating_ip(ip_address=public_ip)
    pulumi.export("fipOutput", example.droplet_id)
    ```


    :param _builtins.str ip_address: The allocated IP address of the specific floating IP to retrieve.
    """
    __args__ = dict()
    __args__['ipAddress'] = ip_address
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('digitalocean:index/getFloatingIp:getFloatingIp', __args__, opts=opts, typ=GetFloatingIpResult)
    return __ret__.apply(lambda __response__: GetFloatingIpResult(
        droplet_id=pulumi.get(__response__, 'droplet_id'),
        floating_ip_urn=pulumi.get(__response__, 'floating_ip_urn'),
        id=pulumi.get(__response__, 'id'),
        ip_address=pulumi.get(__response__, 'ip_address'),
        region=pulumi.get(__response__, 'region')))
